/*
 * Copyright 2009 Ben Axelrod
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the Creative Commons Attribution-Noncommercial-Share
 * Alike 3.0 United States License. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * To view a copy of this license, visit:
 * 
 * http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 *
 * or send a letter to:
 * Creative Commons
 * 171 Second Street, Suite 300
 * San Francisco, California 94105 USA
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CleanMSRS
{
    public partial class Form1 : Form
    {

        List<String> dirTemplates;
        List<String> fileTemplates;
        List<String> toDelete;

        public Form1()
        {
            InitializeComponent();

            pathTextBox.Text = Application.StartupPath;
            searchButton_Click(this, null);
        }

        private void searchButton_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;

            //clear list
            foundTextBox.Text = "";
            foundTextBoxLabel.Text = "Going to delete:";

            //parse dirs and files
            getDirTemplates();
            getFileTemplates();

            traverse(pathTextBox.Text, false);

            if (!String.IsNullOrEmpty(foundTextBox.Text))
            {
                deleteButton.Enabled = true;
                foundTextBox.Enabled = true;
            }

            this.Cursor = Cursors.Default;
        }


        void getDirTemplates()
        {
            dirTemplates = new List<string>();

            String[] temp = dirsTextBox.Text.Split('\n','\r');

            foreach(string s in temp)
            {
                if(!String.IsNullOrEmpty(s))
                    dirTemplates.Add(s);
            }
        }

        void getFileTemplates()
        {
            fileTemplates = new List<string>();

            String[] temp = filesTextBox.Text.Split('\n', '\r');

            foreach (string s in temp)
            {
                if (!String.IsNullOrEmpty(s))
                    fileTemplates.Add(s);
            }
        }

        void getToDelete()
        {
            toDelete = new List<string>();

            String[] temp = foundTextBox.Text.Split('\n', '\r');

            foreach (string s in temp)
            {
                if (!String.IsNullOrEmpty(s))
                    toDelete.Add(s);
            }
        }

        /// <summary>
        /// recursively find files and directories to delete
        /// </summary>
        /// <param name="path"></param>
        /// <param name="deleteDir">whether or not this directory is marked for deletion</param>
        private void traverse(string path, bool deleteDir)
        {
            string[] dirs;
            string[] files;

            try
            {
                dirs = Directory.GetDirectories(path);
                files = Directory.GetFiles(path);
                if (deleteDir)
                {
                    foreach (string f in files)
                        foundTextBox.Text += f + "\r\n";

                    foreach (string d in dirs)
                    {
                        traverse(d, true);
                        foundTextBox.Text += d + "\r\n";
                    }
                }
                else
                {
                    foreach (string f in files)
                    {
                        if (isDeleteFileType(f))
                        {
                            foundTextBox.Text += f + "\r\n";
                        }
                    }

                    foreach (string d in dirs)
                    {
                        if (isDeleteDirType(d))
                        {

                            traverse(d, true);
                            foundTextBox.Text += d + "\r\n";
                        }
                        else
                            traverse(d, false);
                    }
                }
            }
            catch
            {

            }
        }


        private bool isDeleteDirType(string dirpath)
        {
            foreach (string s in dirTemplates)
            {
                if (dirpath.EndsWith(s))
                    return true;
            }
            return false;
        }

        private bool isDeleteFileType(string filepath)
        {
            foreach (string s in fileTemplates)
            {
                if (filepath.EndsWith(s))
                    return true;
            }
            return false;
        }



        private void deleteButton_Click(object sender, EventArgs e)
        {
            getToDelete();

            if (toDelete.Count > 0)
            {
                if (MessageBox.Show(" Are you sure you want to permanently delete all " + toDelete.Count + " files? ",
                    "Confirm Delete",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) == DialogResult.Yes)
                {

                    foundTextBox.Text = "";

                    foreach (String s in toDelete)
                    {
                        try
                        {
                            if (File.GetAttributes(s) == System.IO.FileAttributes.Directory)
                            {
                                try
                                {
                                    processDir(s);
                                }
                                catch
                                {
                                    //add dir back in because an error occurred
                                    foundTextBox.Text += s + "\r\n";
                                }
                            }
                            else // is file
                            {
                                try
                                {
                                    processFile(s);
                                }
                                catch
                                {
                                    //add file back in because an error occurred
                                    foundTextBox.Text += s + "\r\n";
                                }

                            }
                        }
                        catch
                        {
                        }
                    }
                }
            }

            if (!String.IsNullOrEmpty(foundTextBox.Text))
            {
                foundTextBoxLabel.Text = "Unable to delete:";
            }
            else
            {
                deleteButton.Enabled = false;
                foundTextBox.Enabled = false;
            }

        }

        private static void processDir(String s)
        {
            Directory.Delete(s, true);
        }

        private void processFile(String s)
        {
            if (rdonlyCheckBox.Checked)
                File.SetAttributes(s, FileAttributes.Normal);
            File.Delete(s);
        }


        private void pathTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 1) //CTRL-A
            {
                pathTextBox.SelectAll();
            }
        }

        private void dirsTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 1) //CTRL-A
            {
                dirsTextBox.SelectAll();
            }
        }

        private void filesTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 1) //CTRL-A
            {
                filesTextBox.SelectAll();
            }
        }

        private void foundTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 1) //CTRL-A
            {
                foundTextBox.SelectAll();
            }
        }


        private void pathTextBox_DragEnter(object sender, DragEventArgs e)
        {
            // If the data is a file, display the copy cursor.
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void pathTextBox_DragDrop(object sender, DragEventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            try
            {
                if (File.GetAttributes(files[0]) == System.IO.FileAttributes.Directory)
                {
                    pathTextBox.Text = Path.GetFullPath(files[0]);
                }
                else
                {
                    pathTextBox.Text = Path.GetDirectoryName(files[0]);
                }
            }
            catch
            {
            }
            
            this.Cursor = Cursors.Default;
        }


        private void openButton_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fdlg = new FolderBrowserDialog();
            fdlg.SelectedPath = pathTextBox.Text;
            DialogResult result = fdlg.ShowDialog(this);
            
            if (result == DialogResult.OK)
            {
                pathTextBox.Text = fdlg.SelectedPath;
            }
            

        }

    }
}